from typing import Any, Generator, Iterable, Iterator
from .. import Engine

class StreamEngine(Engine):
    def __init__(self, use_stream_api: bool = False, return_as_stream: bool = False):
        self._use_stream_api = use_stream_api
        self._return_as_stream = return_as_stream

    def execute_stream(self, data) -> Generator | Iterator:
        raise NotImplementedError

    def execute_nonstream(self, data) -> Any:
        raise NotImplementedError

    def _execute_stream(self, data: Any, return_as_stream: bool) -> Any:
        results = self.execute_stream(data)
        return results if return_as_stream else list(results)

    def _execute_nonstream(self, data: Any, return_as_stream: bool) -> Any:
        result = self.execute_nonstream(data)
        if return_as_stream:
            def _():
                yield from result
            return _()
        else:
            return result

    def execute(self, data: Any) -> Any:
        if not isinstance(data, Generator) or isinstance(data, bytes):
            if self._use_stream_api:
                return self._execute_stream([data], self._return_as_stream)
            else:
                return self._execute_nonstream(data, self._return_as_stream)
        else:
            if self._use_stream_api:
                if self._return_as_stream:
                    def stream_results():
                        for item in data:
                            yield from self._execute_stream([item], True)
                    return stream_results()
                else:
                    return [self._execute_stream([item], False) for item in data]
            else:
                if self._return_as_stream:
                    def non_stream_results():
                        for item in data:
                            yield self._execute_nonstream(item, False)
                    return non_stream_results()
                else:
                    res = self._execute_nonstream(next(data), False)
                    for item in data:
                        _ = self._execute_nonstream(item, False)
                        if _ is None:
                            continue
                        res += _
                    return res