# https://help.aliyun.com/zh/dashscope/developer-reference/quick-start-7?spm=a2c4g.11186623.0.0.1d5a756e6AcIYc

import json
from typing import Any
from src import ASREngine as ASREngine

from http import HTTPStatus
import dashscope
from dashscope.audio.asr import (Recognition, RecognitionCallback,
                                 RecognitionResult)

from logging import getLogger
logger = getLogger(__name__)

# import requests
# r = requests.get(
# 'https://dashscope.oss-cn-beijing.aliyuncs.com/samples/audio/paraformer/hello_world_female2.wav'
# )
# with open('asr_example.wav', 'wb') as f:
#     f.write(r.content)

class ASRParaformer(ASREngine):
    def __init__(self, api_key : str,
                 _model : str = "paraformer-realtime-v1",
                 _is_stream : bool = False,
                 *args, **kwargs) -> None:
        super().__init__()
        dashscope.api_key = api_key
        self.recognition = Recognition(kwargs.get("model", _model),
                          format='wav',
                          sample_rate=16000,
                          callback=None)

    def process_output(self, data):
        corrected_data = data.replace("'", '"')
        res = json.loads(corrected_data)["text"]
        return super().process_output(res)

    def execute(self, data: Any) -> Any:
        result = self.recognition.call(data)
        if result.status_code == HTTPStatus.OK:
            res = ""
            for sentence in result.get_sentence():
                res += (str(sentence) + '\n')
            return res
        else:
            raise RuntimeError('Error: ' + result.message)