# https://dashscope.console.aliyun.com/model
import asyncio
from concurrent.futures import ThreadPoolExecutor
import json
from typing import Any, AsyncGenerator, AsyncIterator, Generator, Iterator, override
from .remote_engine import RemoteEngine
from src import ExecuteProcessor as ExecuteProcessor

from logging import getLogger
logger = getLogger(__name__)

class ChatAiTongYiQianWen(ExecuteProcessor):
    API_URL = "https://dashscope.aliyuncs.com/api/v1/services/aigc/text-generation/generation"
    def __init__(self, api_key : str,
                 _model : str = "qwen-1.8b-chat",
                 *args, **kwargs) -> None:
        
        self._modual_name = kwargs.get("model", _model)
        self.use_stream_api=kwargs.get("use_stream_api", False)
        self.remote_engine =  RemoteEngine(api_key=api_key,
                                           base_url=self.API_URL)
    def _transform_raw(self, response):
        return response["output"]["choices"][0]["message"]["content"]

    async def _transform_iterator(self, response: AsyncIterator[str]):
        current_msg = ''
        async for line in response:
            if line.startswith("data:"):
                current_msg += line[5:].strip()
            elif len(line) == 0:
                if current_msg:
                    yield self._transform_raw(json.loads(current_msg))
                    current_msg = ''

    async def execute_stream(self, data):
        self.remote_engine.header['X-DashScope-SSE'] = 'enable'
        response = await self.remote_engine.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": True
                }
            })
        return self._transform_iterator(response.aiter_lines())
    
    async def execute_nonstream(self, data) -> Any:
        response = await self.remote_engine.post_prompt({
                'model': self._modual_name,
                "input": {
                    "messages": data
                },
                "parameters": {
                    "result_format": "message",
                    "incremental_output": False
                }
            })
        ret = response.json()
        return self._transform_raw(ret)

    def prepare_input(self, data: Any) -> Any:
        return [{'role': 'user', 'content': f'{data}'}]

    @override
    async def execute(self, data):
        data = self.prepare_input(data)
        executor = ThreadPoolExecutor(max_workers=1)
        if self.use_stream_api:
            res = await self.execute_stream(data)
        else:
            res = await self.execute_nonstream(data)
        return res