from pathlib import Path
import sys


CURRENT_DIR_PATH = Path(__file__).resolve().parent
sys.path.append(str(CURRENT_DIR_PATH.parent.parent.resolve()))
from src import EngineInterface
from .plugins_conf import PluginsConfig
from .plugins_import import PluginsImport

class Plugins:
    def __init__(self,
                 base_path : Path | str = CURRENT_DIR_PATH,
                 conf_name : str = ".plugins.yml"):
        base_path = Path(base_path)
        self.base_path = base_path.resolve()
        self.conf_path = base_path / conf_name

        self.config = PluginsConfig.load_config(self.conf_path)
        # global_conf: Dict = self.config.get('global')
        # self.core_path:Path = self.base_path / (global_conf.get('core_path', '') or '../')
        # self.plugin_path:Path = self.base_path / (global_conf.get('plugin_path', '') or './')
        # sys.path.append(str(self.base_path))

    def _get_engine_import(self, engine_name: str):
        engine_conf, plg_root_path, plg_path = PluginsConfig.get_engine_conf(engine_name, self.base_path, self.conf_path)
        engine_conf.get('plugin_path', '')
        plg_import = PluginsImport(plg_root_path)
        plg_import.get_module(engine_conf['class_name'], plg_path)
        return plg_import, engine_conf

    def load_engine_class(self, engine_name: str):
        plg_import, engine_conf = self._get_engine_import(engine_name)
        return plg_import.get_class(engine_conf['class_name'], engine_conf['class_name'])

    def load_engine(self, engine_name: str) -> EngineInterface:
        plg_import, engine_conf = self._get_engine_import(engine_name)
        return plg_import.get_instance(engine_conf['class_name'], engine_conf['class_name'], **engine_conf)