from pathlib import Path
CURRENT_DIR_PATH = Path(__file__).resolve().parent
import sys
sys.path.append(str(CURRENT_DIR_PATH.parent.parent))

import importlib
from types import ModuleType
from typing import Any

class PluginsImport:
    def __init__(self, plugins_path: Path):
        plugins_path = plugins_path.parent
        if plugins_path not in sys.path:
            sys.path.append(str(plugins_path))
        self.plugins_path = plugins_path
        self.modules = {} # Store loaded modules here

    def get_module(self, module_name: str, module_path: Path) -> ModuleType:
        # Check if the module has already been loaded
        if module_name in self.modules:
            return self.modules[module_name]
        
        # Check if the module path is a subpath of the plugins path
        if not module_path.is_relative_to(self.plugins_path):
            raise ValueError("Module path must be a subpath of the plugins path.")
        
        relative_path_parts  = module_path.relative_to(self.plugins_path).with_suffix('').parts
        relative_path = '.'.join(relative_path_parts)

        try:
            module = importlib.import_module(relative_path, package=self.plugins_path.name)
            self.modules[module_name] = module
            return module
        except ModuleNotFoundError as e:
            raise ModuleNotFoundError(f"Failed to import module '{module_name}' from path '{module_path}': {e}")

    def get_class(self, class_name, module_name) -> Any:
        module = self.modules.get(module_name, None)
        if module is None:
            raise ValueError(f"Module '{module_name}' has not been loaded.")
        cls = getattr(module, class_name, None)
        if cls is None:
            raise AttributeError(f"Class '{class_name}' not found in module '{module_name}'.")
        return cls

    def get_instance(self, _class_name, module_name, **kwargs) -> Any:
        cls = self.get_class(_class_name, module_name)
        return cls(**kwargs)